@@ -113,6 +113,8 @@ KRenameWindow::KRenameWindow( QWidget* parent )
     connect( m_buttonClose, SIGNAL(clicked(bool)), SLOT(close()));
     connect( m_buttons, SIGNAL(accepted()), SLOT(slotFinish()));
 
+    
+    this->setAutoSaveSettings( "KRenameWindowSettings", true );
 
     // Show the first page in any mode
     showPage( 0 );
@@ -122,17 +124,33 @@ KRenameWindow::~KRenameWindow()
 {
 }
 
+void KRenameWindow::loadConfig() 
+{
+    // ExtHistoryCombo needs an object name
+    m_pageFilename->filenameTemplate->setObjectName("FILENAME_TEMPLATE");
+    m_pageFilename->extensionTemplate->setObjectName("EXTENSION_TEMPLATE");
+
+    m_pageFilename->filenameTemplate->loadConfig();
+    m_pageFilename->extensionTemplate->loadConfig();
+}
+
+void KRenameWindow::saveConfig() 
+{
+    m_pageFilename->filenameTemplate->saveConfig();
+    m_pageFilename->extensionTemplate->saveConfig();
+}
+
 void KRenameWindow::setupGui()
 {
     m_buttons->clear();
 
     m_guiMode    = &tAdvancedMode;
 
-    m_buttonFinish = new KPushButton( KIcon( "finish" ), i18n("&Finish...") );
-    m_buttonClose  = new KPushButton( KIcon( "cancel" ), i18n("&Close") );
+    m_buttonFinish = new KPushButton( KIcon( "dialog-ok" ),    i18n("&Finish") );
+    m_buttonClose  = new KPushButton( KIcon( "dialog-close" ), i18n("&Close") );
     
     m_buttons->addButton( m_buttonFinish, QDialogButtonBox::AcceptRole );
-    m_buttons->addButton( m_buttonClose, QDialogButtonBox::RejectRole );
+    m_buttons->addButton( m_buttonClose,  QDialogButtonBox::RejectRole );
 
     setupSlots();
     slotEnableControls();
@@ -170,6 +188,10 @@ void KRenameWindow::setupPlugins()
 
         ++it;
     }
+
+    m_pagePlugins->splitter->setStretchFactor( 0, 0 );
+    m_pagePlugins->splitter->setStretchFactor( 1, 8 );
+    m_pagePlugins->listPlugins->sortColumn();
 }
 
 void KRenameWindow::setupIcons()
@@ -214,7 +236,7 @@ void KRenameWindow::setupSlots()
     connect( m_pageFiles->checkPreview,    SIGNAL(clicked(bool)), SLOT( slotPreviewChanged()));
     connect( m_pageFiles->checkName,       SIGNAL(clicked(bool)), SLOT( slotPreviewChanged()));
     connect( m_pageFiles->comboSort,       SIGNAL(currentIndexChanged(int)), SLOT( slotSortChanged(int)));
-    connect( m_pageFiles->fileList,        SIGNAL(activated(const QModelIndex&)), SLOT( slotOpenFile(const QModelIndex&)));
+    connect( m_pageFiles->fileList,        SIGNAL(doubleClicked(const QModelIndex&)), SLOT( slotOpenFile(const QModelIndex&)));
     connect( m_pageFiles->buttonUp,        SIGNAL(clicked(bool)), SLOT( slotMoveUp() ) );
     connect( m_pageFiles->buttonDown,      SIGNAL(clicked(bool)), SLOT( slotMoveDown() ) );
 
@@ -242,6 +264,7 @@ void KRenameWindow::setupSlots()
     connect( m_pageFilename->buttonDown,         SIGNAL(clicked(bool)), SLOT( slotMoveDownPreview() ) );
 
     connect( m_pageFilename->listPreview,        SIGNAL(addFiles()),    SIGNAL(addFiles()));
+    connect( m_pageFilename->listPreview,        SIGNAL(updateCount()), SIGNAL(updateCount()));
 
     connect( m_pageFilename->comboFilenameCustom,  SIGNAL(delayedTextChanged()), SLOT(slotSimpleTemplateChanged()));
     connect( m_pageFilename->comboSuffixCustom,    SIGNAL(delayedTextChanged()), SLOT(slotSimpleTemplateChanged()));
@@ -279,8 +302,16 @@ void KRenameWindow::showPage( int index )
 
 void KRenameWindow::slotEnableControls()
 {
-    if( m_buttonFinish )
-        m_buttonFinish->setEnabled( m_curPage == m_guiMode->numPages - 1 );
+    // TODO: 
+    // Disable all extension widgets
+    // if splitMode is NOEXTENSION
+
+    if( m_buttonFinish ) 
+    {
+        bool enable = (m_curPage == m_guiMode->numPages - 1)
+            && m_fileCount > 0;
+        m_buttonFinish->setEnabled( enable );
+    }
 
     // enable gui controls
     m_pageFiles->buttonRemove->setEnabled( m_fileCount );
@@ -288,7 +319,6 @@ void KRenameWindow::slotEnableControls()
     m_pageFiles->checkName->setEnabled( m_pageFiles->checkPreview->isChecked() );
    
     m_pageFiles->buttonUp->setEnabled( m_fileCount );
-    m_pageFiles->buttonMove->setEnabled( m_fileCount );
     m_pageFiles->buttonDown->setEnabled( m_fileCount );
 
     m_pageDests->urlrequester->setEnabled( !m_pageDests->optionRename->isChecked() );
@@ -329,6 +359,54 @@ void KRenameWindow::setFilenameTemplate( const QString & templ, bool insert )
         m_pageFilename->filenameTemplate->lineEdit()->setText( templ );
 }
 
+void KRenameWindow::setExtensionTemplate( const QString & templ, bool insert )
+{
+    m_pageFilename->checkExtension->setChecked( true );
+    m_pageFilename->comboExtensionSimple->setCurrentIndex( 
+        m_pageFilename->comboExtensionSimple->count() - 1 );
+
+    if( insert )
+        m_pageFilename->extensionTemplate->lineEdit()->insert( templ );
+    else
+        m_pageFilename->extensionTemplate->lineEdit()->setText( templ );
+}
+
+void KRenameWindow::setRenameMode( ERenameMode eMode ) 
+{
+    m_pageDests->optionRename->setChecked( false );
+    m_pageDests->optionCopy->setChecked( false );
+    m_pageDests->optionMove->setChecked( false );
+    m_pageDests->optionLink->setChecked( false );
+    
+    switch( eMode ) 
+    {
+        case eRenameMode_Rename:
+            m_pageDests->optionRename->setChecked( true );
+            break;
+        case eRenameMode_Copy:
+            m_pageDests->optionCopy->setChecked( true );
+            break;
+        case eRenameMode_Move:
+            m_pageDests->optionMove->setChecked( true );
+            break;
+        case eRenameMode_Link:
+            m_pageDests->optionLink->setChecked( true );
+            break;
+        default:
+            // Default
+            m_pageDests->optionRename->setChecked( true );
+            break;
+    }
+
+    this->slotEnableControls();
+    emit renameModeChanged( eMode );
+}
+
+void KRenameWindow::setDestinationUrl( const KUrl & url )
+{
+    m_pageDests->urlrequester->setUrl( url );
+}
+
 void KRenameWindow::resetFileList() 
 {
     m_pageFiles->fileList->reset();
@@ -396,6 +474,8 @@ bool KRenameWindow::isPreviewEnabled() const
 void KRenameWindow::setPreviewEnabled( bool bPreview )
 {
     m_pageFiles->checkPreview->setChecked( bPreview );
+
+    slotPreviewChanged();
 }
 
 bool KRenameWindow::isPreviewNamesEnabled() const
@@ -406,6 +486,8 @@ bool KRenameWindow::isPreviewNamesEnabled() const
 void KRenameWindow::setPreviewNamesEnabled( bool bPreview )
 {
     m_pageFiles->checkName->setChecked( bPreview );
+
+    slotPreviewChanged();
 }
 
 int KRenameWindow::numberStartIndex() const
@@ -448,6 +530,11 @@ void KRenameWindow::setAdvancedMode( bool bAdvanced )
     m_pageFilename->tabWidget->setCurrentIndex( bAdvanced ? 0 : 1 );
 }
 
+void KRenameWindow::showFilenameTab()
+{
+    m_tabBar->setCurrentIndex( MAX_PAGES - 1 );
+}
+
 void KRenameWindow::setPrefixSuffixSimple( QComboBox* combo, QComboBox* comboCustom, const QString & templ ) 
 {
     if( templ.isEmpty() ) 
@@ -566,7 +653,7 @@ void KRenameWindow::slotSimpleStartIndexChanged()
 {
     emit startIndexChanged( m_pageFilename->spinIndex->value() );
 
-    updatePreview();
+    slotSimpleTemplateChanged();
 }
 
 void KRenameWindow::slotTemplateChanged()
@@ -719,6 +806,9 @@ void KRenameWindow::slotExtensionSplitModeChanged( int index )
         case 1:
             splitMode = eSplitMode_LastDot;
             break;
+        case 2:
+            splitMode = eSplitMode_NoExtension;
+            break;
         default:
             splitMode = eSplitMode_CustomDot;
             break;
@@ -739,6 +829,7 @@ void KRenameWindow::slotPreviewChanged()
     model->setEnablePreview( m_pageFiles->checkPreview->isChecked(), m_pageFiles->checkName->isChecked() );
     emit filePreviewChanged( m_pageFiles->checkPreview->isChecked(), m_pageFiles->checkName->isChecked() );
 
+    m_pageFiles->fileList->setAcceptDrops( true );
     m_pageFiles->fileList->repaint();
     this->slotEnableControls();
 }
@@ -769,8 +860,11 @@ void KRenameWindow::slotSortChanged( int index )
 void KRenameWindow::slotMaxDotsChanged( int dots )
 {
     int i;
+    const int FILE_EXTENSION_VARIABLE_ITEMS_START = 3;
 
-    for( i=2;i<m_pageFilename->comboExtension->count();i++ )
+    for( i=FILE_EXTENSION_VARIABLE_ITEMS_START;
+         i<m_pageFilename->comboExtension->count();
+         i++ )
         m_pageFilename->comboExtension->removeItem( i );
 
     for( i=1;i<=dots;i++ )
